/*
 * (C)opyright MMIV-MMV Anselm R. Garbe <garbeam at gmail dot com>
 * See LICENSE file for license details.
 */

#ifndef _DAT_H
#define _DAT_H

#include <stdio.h>
#include <X11/Xlib.h>
#include <X11/Xutil.h>

#include "ixp.h"
#include "wmii.h"

/* Motif bits - see Motif reference manual for further information */
#define MWM_HINTS_DECORATIONS   (1L << 1)

struct MWMHints {
    long flags;
    long funcs;
    long decor;
    int input;
    long state;
};

/* file pointer array indexes of frame file pointers */
#define F_PREFIX                0
#define F_CTL                   1
#define F_NAME                  2
#define F_LAYOUT                3
#define F_SIZE                  4
#define F_BORDER_W              5
#define F_TAB_H                 6
#define F_HANDLE_INC            7
#define F_IGNORE_LAYOUT         8
#define F_SELECTED_BG_COLOR     9
#define F_SELECTED_TEXT_ALIGN   10
#define F_SELECTED_TEXT_FONT    11
#define F_SELECTED_TEXT_SIZE    12
#define F_SELECTED_FG_COLOR     13
#define F_SELECTED_BORDER_COLOR 14
#define F_NORMAL_BG_COLOR       15
#define F_NORMAL_TEXT_ALIGN     16
#define F_NORMAL_TEXT_FONT      17
#define F_NORMAL_TEXT_SIZE      18
#define F_NORMAL_FG_COLOR       19
#define F_NORMAL_BORDER_COLOR   20
#define F_EVENT_B1CLICK         21
#define F_EVENT_B2CLICK         22
#define F_EVENT_B3CLICK         23
#define F_EVENT_B4CLICK         24
#define F_EVENT_B5CLICK         25
#define F_SELECTED_FRAME        26
#define F_FRAME_PREFIX          27
#define F_LAST                  28

#define DEFAULT_BORDER_W        "3"
#define DEFAULT_TAB_H           "16"

/* file pointer array indexes of client file pointers */
#define C_NAME                  0
#define C_CLASS                 1
#define C_INSTANCE              2
#define C_LAST                  3

#define CORE_CTL                0
#define CORE_FRAME_SEL          1
#define CORE_DETACHED           2
#define CORE_ACTION_LOOKUP      3
#define CORE_TRANS_COLOR        4
#define CORE_SNAP_VALUE         5
#define CORE_SELECTED_PAGE      6
#define CORE_EVENT_PAGE_CREATE  7
#define CORE_EVENT_PAGE_SEL     8
#define CORE_EVENT_PAGE_REQ     9
#define CORE_EVENT_PAGE_UPDATE  10
#define CORE_LAST               11

#define PROTO_DEL               1

typedef struct Cell Cell;
typedef struct Layout Layout;
typedef struct Frame Frame;
typedef struct Client Client;

struct Cell {
    int x, y, w, h;
};

struct Layout {
    int cells;
    Cell *cell;
    Layout *next;
};

struct Frame {
    int         id;
    int         num_act;
    int         decor;
    Window      win;
    GC          gc;
    GC          xorgc;
    XRectangle  rect;
    Cursor      cursor;
    Action      *acttbl;
    Ring        *sel;
    Ring        *childs;
    File        *files[F_LAST];
    Frame       *parent;
    Client      *client;
    Layout      *layout;
    Cell        *relcell;
    Ring        *frames; /* only used in pages */
};

struct Client {
    int             state;
    int             proto;
    int             reparenting;
    unsigned int    border;
    Window          win;
    Window          trans;
    XRectangle      rect;
    XSizeHints      size;
    Frame           *frame;
    File            *files[C_LAST];
};

/* global variables */
extern Display *dpy;
extern char *ds;
extern IXPServer *ixps;
extern int mon;
extern int fid;
extern int cid;
extern Window root;
extern XRectangle rect;
extern Ring *detached;
extern Ring *pages, *first_page; /* pages */
extern Frame *zoomed;
extern XRectangle zoom_rect;
extern XColor xorcolor;
extern Client *sel;

extern Atom wm_state;
extern Atom wm_change_state;
extern Atom wm_protocols;
extern Atom wm_delete;
extern Atom motif_wm_hints;
extern Atom net_wm_desktop;

extern Cursor normal_cursor;
extern Cursor resize_cursor;
extern Cursor move_cursor;
extern Cursor drag_cursor;
extern Cursor w_cursor;
extern Cursor e_cursor;
extern Cursor n_cursor;
extern Cursor s_cursor;
extern Cursor nw_cursor;
extern Cursor ne_cursor;
extern Cursor sw_cursor;
extern Cursor se_cursor;

/* default file pointers */
extern File *defaults[F_LAST];
extern File *core_files[CORE_LAST];

#endif /* _DAT_H */
