/*
 * (C)opyright MMIV-MMV Anselm R. Garbe <garbeam at gmail dot com>
 * See LICENSE file for license details.
 */

#ifndef _FNS_H
#define _FNS_H

#include "wmii.h"

/* client.c */
Client *alloc_client(Frame *parent, Window w, XWindowAttributes *wa);
void free_client(Client *c);
void configure_client(Client *c);
void handle_client_property(Client *c, XPropertyEvent *e);
void close_client(Client *c);
void gravitate(Client *c, unsigned int tabh, unsigned int bw, int invert);
void invoke_class_instance_hook(Client *c, char *hook);

/* core.c */
void invoke_page_event(File *f);
void run_action(File *f);
void _exec(void *, char *cmd);
Align cursor_to_align(Cursor cursor);
void scan_wins();
Client *win_to_client(Ring *root, Window w);
Frame *win_to_frame(Ring *root, Window w);
int win_proto(Window w);
int win_decor(Window w);
int win_state(Window w);
void set_win_state(Client *c, int state);
void handle_after_write(IXPServer *s, File *f);
void focus(Frame *f, int raise);
void grab_client(Client *c);
void ungrab_client(Client *c);
void attach(Frame *parent, Frame *child);
void detach(Frame *f, int client_destroyed);
int comp_obj(void *f1, void *f2);
Frame *top_frame(Frame *child);
void invoke_rc(char *mode, void (*spawn_func)(void *, char *, char*));

/* frame.c */
Frame *alloc_frame(Frame *parent, XRectangle *r, int add_frame_border, int decor);
void free_frame(Frame *f);
void resize(Frame *f);
void draw_frame(Frame *f);
void draw_child(Frame *f);
void handle_frame_buttonpress(XButtonEvent *e, Frame *f);
Cursor cursor_for_xy(Frame *f, int x, int y);
int size_of_frame(Frame *f);
void attach_frame_to_frame(Frame *parent, Frame *child);
void detach_frame_from_frame(Frame *child);
Frame *root_frame(Frame *f);
void show_page(Frame *f);
void hide_page(Frame *f);
void set_ignore_layout(Frame *f, char *flag);

/* event.c */
void init_event_hander();
void check_event(Connection *c);

/* layout.c */
Layout *alloc_layout(char *lstr);
Layout *match_layout(Layout *layout, int nclients);
Frame *apply_layout(Frame *f);
void free_layout(Layout *l);
void arrange(Frame *f, int tabh, int bw);
Cell **neighbors(Layout *l, Cell *o, Align align, int *res_size);

#endif /* _FNS_H */
