#!/usr/bin/env python

################################################################################
# Copyright (c) 2005 Georg Neis <georg@f-451.net>
#
# Permission is hereby granted, free of charge, to any person obtaining a copy
# of this software and associated documentation files (the "Software"), to deal
# in the Software without restriction, including without limitation the rights
# to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
# copies of the Software, and to permit persons to whom the Software is
# furnished to do so, subject to the following conditions:
#
# The above copyright notice and this permission notice shall be included in all
# copies or substantial portions of the Software.
#
# THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
# IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
# FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
# AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
# LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
# OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE
# SOFTWARE.
################################################################################


"""
This tool uses ixplib and wmii_expand to expand all the wmii actions
that occur in the given string.  The result is written to stdout.

Examples:
    % expand_actions.py '@{quit}'
    wmir write /wmii/ctl quit
    % expand_actions.py 'foobar'
    foobar
"""


import os
import sys

import ixplib
import wmii_expand


if __name__ == '__main__':
    if len(sys.argv) != 2:
        print >>sys.stderr, "usage: %s <string>" % sys.argv[0]
        print >>sys.stderr, __doc__,
        sys.exit(1)

    client1 = ixplib.Client('%s/.wmii/ixp/wmifs-%s' % (
                            os.getenv('HOME'), os.getenv('DISPLAY')))
    fd = client1.open('/keys/lookup/actions')
    location, socket2 = client1.read(fd).split()
    client1.close(fd)
    client2 = ixplib.Client(socket2)
    print wmii_expand.expand_actions(client2, location, sys.argv[1])
