################################################################################
# Copyright (c) 2005 Georg Neis <georg@f-451.net>
#
# Permission is hereby granted, free of charge, to any person obtaining a copy
# of this software and associated documentation files (the "Software"), to deal
# in the Software without restriction, including without limitation the rights
# to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
# copies of the Software, and to permit persons to whom the Software is
# furnished to do so, subject to the following conditions:
#
# The above copyright notice and this permission notice shall be included in all
# copies or substantial portions of the Software.
#
# THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
# IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
# FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
# AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
# LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
# OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE
# SOFTWARE.
################################################################################


"""Useful wmii related functions."""


import re

import ixplib


class Error(Exception):
    pass


def _test():
    import doctest
    doctest.testmod()

def contains_action(s):
    """Check if string s contains one or more action variables.

    >>> contains_action('@{foobar')
    False
    >>> contains_action('@{}')
    False
    >>> contains_action('@{x}')
    True
    >>> contains_action('@{adfasfd}; @{fdsadfasd}dfasd')
    True
    """
    return bool(re.search('@{.+}', s))

def expand_actions(client, actionsdir, s, reclvl=20):
    """Expand all actions that occur in string s and return the result.

    Arguments:
    client -- ixplib.Client connected to the socket where actionsdir resides
    actionsdir -- pathname of the directory that contains the actions
    s -- string that contains zero or more actions that shall be expanded
    reclvl -- number of recursive calls after which an exception is thrown

    Exceptions:
    Error is raised when an action file could not be opened/read
    or when the number of recursive calls is greater than reclvl.
    """
    try:
        if reclvl <= 0:
            raise Error('Too many levels of recursion.')
        at_pos = s.rindex('@{')
        rbrace_pos = s.index('}', at_pos+2)
        action = s[at_pos+2:rbrace_pos]
        fd = client.open(actionsdir + '/' + action)
        expansion = client.read(fd)
        client.close(fd)
        return expand_actions(client, actionsdir,
                              s[:at_pos] + expansion + s[rbrace_pos+1:],
                              reclvl - 1)
    except ValueError:
        return s
    except ixplib.ClientError, e:
        raise Error(e.value)


if __name__ == '__main__':
    _test()
