/*
 * (C)opyright MMV Anselm R. Garbe <garbeam at gmail dot com>
 * See LICENSE file for license details.
 */

#include <stdio.h>
#include <string.h>
#include <stdlib.h>

#include "ixp.h"

#define MAX_RECURSION_DEPTH 100

static char *get_start(char *s) {

    char *t = strchr(s, '@');
    if (!t) return 0;
    t++;
    if (*t != '{') return get_start(t);
    t++;
    return t;
}

char *expand_action(char *sockfile, char *action_dir, char *glob, unsigned int depth) {

    IXPClient *c = init_client(sockfile);
    char *tmp, *s1, *s2;
    char path[256];
    char ch;
    char *result = 0;
    size_t len, out_len;
    char buf[2050];
    int fd = -1;

    if (!c) return 0;
    if (!action_dir || !glob) {
        deinit_client(c);
        return depth == 0 ? strdup(glob) : 0;
    }

    /* endless recursion prevention */
    if (depth > MAX_RECURSION_DEPTH) {
        deinit_client(c);
        fprintf(stderr,
                "libwmii: warning, interrupting expand_action() with glob='%s'\n"
                "         at maximum recursion depth: %d\n", glob,
                MAX_RECURSION_DEPTH);
        return 0;
    }

    /* first lookup for expansion stuff */
    /*printf("start: %s\n", glob);*/
    s1 = get_start(glob);
    if (!s1) {
       deinit_client(c);
       return depth == 0 ? strdup(glob) : 0;
    }
    s2 = strchr(s1, '}');
    if (!s2) {
       deinit_client(c);
       return depth == 0 ? strdup(glob) : 0;
    }

    /* next create action file path to expand content from */
    ch = *s2;
    *s2 = '\0';
    snprintf(path, 256, "%s/%s", action_dir, s1);
    *s2 = ch;

    s1 -= 2;
    ch = *s1;
    *s1 = '\0';
    len = strlen(glob) + 1;
    result = malloc(len);
    STRLCPY(result, glob, len);

    /* fetch file contents to expand */
    fd = c->open(c, path);
    if (c->errstr) {
        deinit_client(c);
        *s1 = ch;
        fprintf(stderr, "libwmii: expand_action(): %s (%s)\n",
                c->errstr, path);
        return result;
    }
    do {
        out_len = c->read(c, fd, buf, 2048);
        if (c->errstr) {
            deinit_client(c);
            *s1 = ch;
            fprintf(stderr, "libwmii: expand_action(): %s (%s)\n",
                    c->errstr, buf);
            return result;
        }
        len += out_len;
        tmp = malloc(len + out_len);
        STRLCPY(tmp, result, len + out_len);
        buf[out_len] = '\0';
        STRLCAT(tmp, buf, len + out_len);
        len += out_len;
        free(result);
        result = tmp;
    }
    while (out_len == 2048);
    c->close(c, fd);
    *s1 = ch;
    if (c->errstr)
        fprintf(stderr, "libwmii: expand_action(): %s\n", c->errstr);
    deinit_client(c);

    /* pad the rest to expand */
    s2++;
    if (*s2 != '\0') {
        size_t size = len + strlen(s2);
        tmp = malloc(len + size);
        STRLCPY(tmp, result, size);
        STRLCAT(tmp, s2, size);
        free(result);
        result = tmp;
    }

    /*printf("exit: %s\n", result);*/
    /* process recursively */
    tmp = expand_action(sockfile, action_dir, result, depth + 1);
    if (tmp) {
        free(result);
        result = tmp;
    }
    return result;
}
