/*
 * (C)opyright MMIV-MMV Anselm R. Garbe <garbeam at gmail dot com>
 * See LICENSE file for license details.
 */

#include <stdlib.h>

#include "wmii.h"

static Ring zero_ring = {0};

Ring *r_attach(Ring *r, void *item) {
    Ring *new = malloc(sizeof(Ring));
    *new = zero_ring;
    new->item = item;
    if (r) {
        new->prev = r->prev;
        new->prev->next = new;
        r->prev = new;
        new->next = r;
    }
    else {
        new->next = new->prev = new;
    }
    return new;
}

void free_ring(Ring *r, void (*free_item)(void *)) {

    Ring *n, *p = r;
    if (!r) return;

    do {
        n = p->next;
        free_item(p->item);
        free(p);
        p = n;
    } while (p != r);
}

/* enables that a ring can be used as cyclic stack */
Ring *r_top(Ring *r, void *pattern, int (*comp)(void *, void *)) {

    Ring *tmp = r_find(r, pattern, comp);
    if (tmp) {
        r_swap(r, tmp);
        return tmp;
    }
    return 0;
}

Ring *r_find(Ring *r, void *pattern, int (*comp)(void *, void *)) {

    Ring *p = r;
    for (; p; p = p->next) {
        if (comp(pattern, p->item)) return p;
        if (p->next == r) break; /* full loop */
    }
    return 0;
}

Ring *r_detach(Ring *r, void *item) {
    Ring *p = r;
    for (; p; p = p->next) {
        if (p->item == item) {
            if (p->prev != p) {
                p->prev->next = p->next;
                p->next->prev = p->prev;
            }
            if (p->next == p) r = 0;
            if (p == r) r = r->next;
            free(p);
            break;
        }
        if (p->next == r) break; /* full loop */
    }
    return r;
}

void r_swap(Ring *r1, Ring *r2) {

    Ring *tmp;
    if (r1 == r2) return;
    tmp = r1->prev;
    r1->prev = r2->prev;
    r1->prev->next = r1;
    r2->prev = tmp;
    r2->prev->next = r2;
    tmp = r2->next;
    r2->next = r1->next;
    r2->next->prev = r2;
    r1->next = tmp;
    r1->next->prev = r1;
}
