/*
 * (C)opyright MMIV-MMV Anselm R. Garbe <garbeam at gmail dot com>
 * See LICENSE file for license details.
 */

#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <unistd.h>
#include <X11/Xlib.h>
#include <X11/Xatom.h>
#include <X11/Xutil.h>

#include "wmii.h"

int property(Display *dpy, Window w, Atom a, Atom t, long l, unsigned char **prop) {
    Atom real;
    int format;
    unsigned long res, extra;
    int status;

    status = XGetWindowProperty(dpy, w, a, 0L, l, False, t, &real, &format,
                                &res, &extra, prop);

    if (status != Success || *prop == 0) {
        return 0;
    }
    if (res == 0) {
        free((void*) *prop);
    }

    return res;
}

void win_prop(Display *dpy, Window w, Atom a, char *res, int len) {

    unsigned char *prop;

    if (property(dpy, w, a, XA_STRING, 100L, &prop)) {
        STRLCPY(res, (char *)prop, len);
        XFree(prop);
    }
    XSync(dpy, False);
}

void install_cursor(Display *dpy, Window w, Cursor c) {
    XSetWindowAttributes wa;
    wa.cursor = c;
    XChangeWindowAttributes(dpy, w, CWCursor, &wa);
    XSync(dpy, False); /* take effect immediately */
}

void send_message(Display *dpy, Window w, Atom a, long value) {

    XEvent e;
    e.type = ClientMessage;
    e.xclient.window = w;
    e.xclient.message_type = a;
    e.xclient.format = 32;
    e.xclient.data.l[0] = value;
    e.xclient.data.l[1] = CurrentTime;

    XSendEvent(dpy, w, False, NoEventMask, &e);
    XFlush(dpy);
}

char *display_string(Display *dpy, int screen_num) {

    char *ds, *colon, *dot;
    char dstr[512];

    ds = DisplayString(dpy);
    STRLCPY(dstr, "DISPLAY=", sizeof(dstr));
    STRLCAT(dstr, ds, sizeof(dstr));
    colon = strrchr(ds, ':');
    if (colon) {
        colon = dstr + 8 + (colon - ds);
        dot = strchr(colon, '.');
        if (!dot) dot = colon + strlen(colon);
        snprintf(dot, 3, ".%d", screen_num);
    }
    return strdup(dstr);
}
