#!/usr/bin/env ruby
=begin
  Proof of concept toggled clientbar for wmibar by zahod.
  Right click on the status label or on any of the client labels to toggle.

  1) change your bar setup, for example make ~/.wmii/bar look like this:
     . "$GLOBALCONFDIR/bar"
     $WRITE /bar/label/status/event/b3click "wmir write /bar/lookup/labels /clientbar"
  2) put toggled_clientbar.rb in ~/.wmii
  3) type 'wmir write /wmii/event/page-update "$HOME/.wmii/./toggled_clientbar.rb"'
     while running wmii

  TODO:
    display an empty label when all clients are closed
    learn how to do C bindings - this could use some speed :)
      then display selected client as focused

  This script is in the Public Domain.
=end

def read(path)
	`wmir read #{path}`
end

def read_strip(path)
	`wmir read #{path}`.strip
end

def write(path, data)
	system("wmir write #{path} \"#{data}\"")
end

def remove(path)
	system("wmir remove #{path}")
end

def write_defaults(frame)
	write("/bar/ctl", "new /clientbar/#{frame}")
	write("/bar/clientbar/#{frame}style/text-align", STYLE[:text_align])
	write("/bar/clientbar/#{frame}style/text-font", STYLE[:text_font])
	write("/bar/clientbar/#{frame}style/text-size", STYLE[:text_size])
	write("/bar/clientbar/#{frame}style/text-color", STYLE[:fg_color])
	write("/bar/clientbar/#{frame}style/bg-color", STYLE[:bg_color])
	write("/bar/clientbar/#{frame}style/border-color", STYLE[:border_color])
	write("/bar/clientbar/#{frame}event/b1click", "wmir write /wmii/ctl 'select #{frame.chop}'")
	write("/bar/clientbar/#{frame}event/b3click", "wmir write /bar/lookup/labels /label")
end


STYLE = {
:height => read_strip("/wmii/default/frame/tab-height"),
:text_align => read_strip("/wmii/default/frame/norm-style/text-align"),
:text_font => read_strip("/wmii/default/frame/norm-style/text-font"),
:text_size => read_strip("/wmii/default/frame/norm-style/text-size"),
:fg_color => read_strip("/wmii/default/frame/norm-style/text-color"),
:bg_color => read_strip("/wmii/default/frame/norm-style/bg-color"),
:border_color => read_strip("/wmii/default/frame/norm-style/border-color")
}


old_labels = read("/bar/clientbar/").split(/\n/)


def process(dir)
	data = read(dir + "frame/")
	if !(data == "sel/\n" or data == "sel\n\n")
		data.split(/\n/).each { |frame|
			if !(frame == "sel/")
				process(dir + "frame/" + frame)
				if !(read(dir + "frame/#{frame}client") == "") 
					@frames.push(frame) #get frame ids
					@clients.push(read_strip(dir + "frame/#{frame}client/name")) #get client names
				end
			end
		}
	end
end

pages = read("/wmii/page/").split(/\n/) #get page ids
pages.delete("sel/")
@frames = []
@clients = []
pages.each { |page|
	process("/wmii/page/" + page)
}


x_dimension = `xdpyinfo |grep dimension`.split(/ /)[6].split(/x/)[0]
width = ((x_dimension.to_f) / @clients.length).round
height = STYLE[:height]


=begin debug
puts "Pages (#{pages.length}):"
puts pages
puts "Frames (#{@frames.length}):"
puts @frames
puts "Clients (#{@clients.length}):"
puts @clients
puts "Screen x dimension: " + x_dimension
puts "Clientbar dimension: #{width * @clients.length + (x_dimension.to_i - width * @clients.length)}x#{height}"
=end


clientcount = @clients.length - 1
0.upto(clientcount) { |num|
	if num != clientcount
		labelwidth = width
	else
		labelwidth = width + (x_dimension.to_i - width * @clients.length)
	end
	write_defaults(@frames[num])
	write("/bar/clientbar/#{@frames[num]}size", "#{width * num},0,#{labelwidth},#{height}")
	write("/bar/clientbar/#{@frames[num]}text", @clients[num])
}


#remove potentially old and unused labels
old_labels.each { |frame|
	remove("/bar/clientbar/#{frame}") unless @frames.include? frame
}
